# This model is published on: Teunis, P. F., Marinović, A. B., Tribble, D. R., Porter, C. K., & Swart, A. (2018). Acute illness from Campylobacter jejuni may require high doses while infection occurs at low doses. Epidemics, 24, 1-20.
# Please, when applying the model, make reference to the paper above.

# libraries
library(MASS)
library(gsl)

####################################
##### Hypergeometric functions #####
####################################

### 'hg.r' (Teunis et al., 2018) ###

#
drbp <- function(alpha,beta,cV) return(1-(1+cV/beta)^(-alpha))

#
dr1f1 <- function(alpha,beta,cV) {
  if(cV<1e-4) return((alpha/(alpha+beta))*cV)
  if(alpha>1e3 && beta < alpha/100) return(1-exp(-cV))
  if(alpha>1e2 && beta>1e5) return(1-exp(-cV*alpha/beta))
  if(alpha>1e1 && beta>1e5 && cV*alpha/beta>10) return(drbp(alpha,beta,cV))
  if(alpha>1 && beta>alpha*20 && cV>10) return(drbp(alpha,beta,cV))
  if(alpha>1 && beta>alpha && cV>50) return(drbp(alpha,beta,cV))
  if(alpha>1 && beta<alpha && cV>20) return(drbp(alpha,beta,cV))
  if(alpha<1 && beta>alpha*50) return(drbp(alpha,beta,cV))
  if(alpha<0.1 && beta>alpha*20) return(drbp(alpha,beta,cV))
  if(abs(round(alpha)-alpha)<1e-4) alpha=1.0001*alpha
  if(abs(round(beta)-beta)<1e-4) beta=1.0001*beta
  return(1-hyperg_1F1(alpha,alpha+beta,-cV))
}

#
pfaff2f1 <- function(a,b,c,z) {
  if(abs(z)>1) return(hyperg_2F1(c-a,b,c,z/(z-1))*(1-z)^(-b))
  return(hyperg_2F1(a,b,c,z))
}

#
dr2f1pg <- function(alpha,beta,rho,cV) {
  if(rho>10)return (dr1f1(alpha,beta,cV))
  return(1-pfaff2f1(alpha,rho,alpha+beta,-cV/rho))
}

#
muc <- function(a) return(-a/((1-a)*log(1-a)))

#
dr2f1aga <- function(alpha,beta,a,cV) {
  agg <- muc(a)
  if(-cV/log(1-a)>10) return(dr1f1(alpha,beta,cV*agg))
  return(1-pfaff2f1(alpha,-cV/log(1-a),alpha+beta,-a/(1-a)))
}

#
dr2f1agv <- function(alpha,beta,a,cV) {
  agg <- muc(a)
  if(-cV/(agg*log(1-a))>10) return(dr1f1(alpha,beta,cV))
  return(1-pfaff2f1(alpha,-cV/(agg*log(1-a)),alpha+beta,-a/(1-a)))
}

#
drinf <- function(a,b,cV) dr1f1(a,b,cV)

#
drill <- function(r,eta,cV) 1-(1+cV/eta)^(-r)

#
pmdist <- function(a,b,u) {
  norm <- gamma(a+b)/(gamma(a)*gamma(b))
  return(exp(u*a)/((1+exp(u))^(a+b))*norm)
}

#
target.inf <- function(logcV,a,b,q) return(drinf(a,b,exp(logcV))-q)

#
target.ill <- function(logcV,a,b,r,eta,q) return(drinf(a,b,exp(logcV))*drill(r,eta,exp(logcV))-q)

#
findinfdq <- function(q,a,b) {
  if(sign(target.inf(-6,a,b,q))==sign(target.inf(25,a,b,q))) return(NA)
  result <- uniroot(target.inf,a=a,b=b,q=q,lower=-6,upper=25)
  return(exp(result$root))
}

#
findilldq <- function(q,a,b,r,eta) {
  if(sign(target.ill(-6,a,b,r,eta,q))==sign(target.ill(25,a,b,r,eta,q))) return(NA)
  result <- uniroot(target.ill,a=a,b=b,r=r,eta=eta,q=q,lower=-6,upper=25)
  return(exp(result$root))
}

#################
##### Model #####
#################

### Adapted from 'stats.r' (Teunis et al., 2018) ###

#mean, median, and 2.5% and 97.5% percentiles
calcstats <- function(x){
  x.mn <- mean(x, na.rm=T)
  x.qn <- quantile(x,c(0.5,0.025,0.975), na.rm=T)
  return(c("mean"=x.mn, x.qn))
}

TeunisDR <- function(n_sim, dose, Pexp, challenge, 
                     mean_w_inf_ch, mean_z_inf_ch, var_w_inf_ch, var_z_inf_ch, cov_wz_inf_ch, 
                     mean_w_ill_ch, mean_z_ill_ch, var_w_ill_ch, var_z_ill_ch, cov_wz_ill_ch, 
                     mean_w_inf_ob, mean_z_inf_ob, var_w_inf_ob, var_z_inf_ob, cov_wz_inf_ob, 
                     mean_w_ill_ob, mean_z_ill_ob, var_w_ill_ob, var_z_ill_ob, cov_wz_ill_ob){
  
  ### Challenge ###
  
  #Means of w1/z1
  mean.inf.challenge <- c(mean_w_inf_ch, mean_z_inf_ch)
  
  #Covariance matrix for of (w1,z1)
  cov.inf.challenge <- matrix(c(var_w_inf_ch, cov_wz_inf_ch, 
                                cov_wz_inf_ch, var_z_inf_ch), 2)
  
  #Means of w2/z2
  mean.ill.challenge <- c(mean_w_ill_ch, mean_z_ill_ch)
  
  #Covariance matrix for of (w2,z2)
  cov.ill.challenge <- matrix(c(var_w_ill_ch, cov_wz_ill_ch, 
                                cov_wz_ill_ch, var_z_ill_ch), 2)
  
  ### Outbreak ###
  
  #Means of w1/z1
  mean.inf.outbreak <- c(mean_w_inf_ob, mean_z_inf_ob)
  
  #Covariance matrix for of (w1,z1)
  cov.inf.outbreak <- matrix(c(var_w_inf_ob, cov_wz_inf_ob, 
                               cov_wz_inf_ob, var_z_inf_ob), 2)
  
  #Means of w2/z2
  mean.ill.outbreak <- c(mean_w_ill_ob, mean_z_ill_ob)
  
  #Covariance matrix for of (w2,z2)
  cov.ill.outbreak <- matrix(c(var_w_ill_ob, cov_wz_ill_ob, 
                               cov_wz_ill_ob, var_z_ill_ob), 2)
  
  if(challenge){ #challenge
    #Generation of multivariate normal samples for w1 and z1
    w1z1 <- mvrnorm(n_sim, mu=mean.inf.challenge, Sigma=cov.inf.challenge, empirical = T)
    colnames(w1z1) <- c('w1', 'z1')
    w1z1 <- as.data.frame(w1z1)
    
    #Generation of multivariate normal samples for w2 and z2
    w2z2 <- mvrnorm(n_sim, mu=mean.ill.challenge, Sigma=cov.ill.challenge, empirical = T)
    colnames(w2z2) <- c('w2', 'z2')
    w2z2 <- as.data.frame(w2z2)
  } else { #outbreak
    #Generation of multivariate normal samples for w1 and z1
    w1z1 <- mvrnorm(n_sim, mu=mean.inf.outbreak, Sigma=cov.inf.outbreak, empirical = T)
    colnames(w1z1) <- c('w1', 'z1')
    w1z1 <- as.data.frame(w1z1)
    
    #Generation of multivariate normal samples for w2 and z2
    w2z2 <- mvrnorm(n_sim, mu=mean.ill.outbreak, Sigma=cov.ill.outbreak, empirical = T)
    colnames(w2z2) <- c('w2', 'z2')
    w2z2 <- as.data.frame(w2z2)
  }
  
  ### Infection ###
  
  #Parameter transformation
  u1v1 <- matrix(NA, n_sim, 2)
  colnames(u1v1) <- c('u1', 'v1')
  u1v1 <- as.data.frame(u1v1)
  u1v1$u1 <- exp(w1z1$w1)/(1+exp(w1z1$w1))
  u1v1$v1 <- exp(w1z1$z1)
  
  #Calculation of alpha and beta
  alphabeta <- matrix(NA, n_sim, 2)
  colnames(alphabeta) <- c('alpha', 'beta')
  alphabeta <- as.data.frame(alphabeta)
  alphabeta$alpha <- u1v1$u1*u1v1$v1
  alphabeta$beta <- (1-u1v1$u1)*u1v1$v1
  
  alpha.par <- alphabeta$alpha
  beta.par <- alphabeta$beta
  
  #Summary for alpha and beta
  alphabeta.sum <- matrix(NA, 2, 4)
  rownames(alphabeta.sum) <- c('alpha', 'beta') 
  colnames(alphabeta.sum) <- c('mean', 'median', '2.5%', '97.5%')
  alphabeta.sum[1,] <- calcstats(alpha.par)
  alphabeta.sum[2,] <- calcstats(beta.par)

  ### Illness ###
  
  #Parameter transformation
  u2v2 <- matrix(NA, n_sim, 2)
  colnames(u2v2) <- c('u2', 'v2')
  u2v2 <- as.data.frame(u2v2)
  u2v2$u2 <- exp(w2z2$w2)/(1+exp(w2z2$w2))
  u2v2$v2 <- exp(w2z2$z2)
  
  #Calculation of r and eta
  reta <- matrix(NA, n_sim, 2)
  colnames(reta) <- c('r', 'eta')
  reta <- as.data.frame(reta)
  reta$r <- u2v2$u2*u2v2$v2
  reta$eta <- (1-u2v2$u2)*u2v2$v2
  
  r.par <- reta$r
  eta.par <- reta$eta
  
  #Summary for r and eta
  reta.sum <- matrix(NA, 2, 4)
  rownames(reta.sum) <- c('r', 'eta')
  colnames(reta.sum) <- c('mean', 'median', '2.5%', '97.5%')
  reta.sum[1,] <- calcstats(reta$r)
  reta.sum[2,] <- calcstats(reta$eta)
  
  ### Probability of illness ###
  
  n_dose <- length(dose)
  
  #Probability of infection
  Pinfexp <- matrix(NA, n_sim, n_dose)
  for (k in 1:n_sim){
    for (d in 1:n_dose) Pinfexp[k,d] <- drinf(alpha.par[k], beta.par[k], dose[d])
  } 
  Pinf <- Pinfexp*Pexp
  
  #Mean probability of infection over doses
  Qinfmean <- matrix(NA, n_sim, 1)
  for (k in 1:n_sim) Qinfmean[k] <- mean(Pinf[k,])
  
  #Conditional probability of illness
  Pillinf <- matrix(NA, n_sim, n_dose)
  for (k in 1:n_sim){
    for (d in 1:n_dose) Pillinf[k,d] <- drill(r.par[k], eta.par[k], dose[d])
  } 
  
  #Probability of illness
  Qill <- Pillinf*Pinf
  
  #Mean probability of illness over doses
  Qillmean <- matrix(NA, n_sim, 1)
  for (k in 1:n_sim) Qillmean[k] <- mean(Qill[k,])

  # output
  drlist = list(Qinfmean=Qinfmean, Qillmean=Qillmean, dose = dose, n_sim=n_sim)
}

#challenge
runDR <- TeunisDR(n_sim, dose, Pexp, challenge, 
                  mean_w_inf_ch, mean_z_inf_ch, var_w_inf_ch, var_z_inf_ch, cov_wz_inf_ch, 
                  mean_w_ill_ch, mean_z_ill_ch, var_w_ill_ch, var_z_ill_ch, cov_wz_ill_ch, 
                  mean_w_inf_ob, mean_z_inf_ob, var_w_inf_ob, var_z_inf_ob, cov_wz_inf_ob, 
                  mean_w_ill_ob, mean_z_ill_ob, var_w_ill_ob, var_z_ill_ob, cov_wz_ill_ob)

Qinfmean <- runDR$Qinfmean
Qillmean <- runDR$Qillmean